@@ -4,7 +4,7 @@
  * darcy@druid.net
  * http://www.druid.net/darcy/
  *
- * $Id: chkpass.c,v 1.11.2.1 2005/01/29 22:35:29 tgl Exp $
+ * $PostgreSQL: pgsql/contrib/chkpass/chkpass.c,v 1.12 2003/11/29 22:39:18 pgsql Exp $
  * best viewed with tabs set to 4
  */
 
@@ -82,10 +82,14 @@ chkpass_in(PG_FUNCTION_ARGS)
 	}
 
 	if (verify_pass(str) != 0)
+	{
 		ereport(ERROR,
 				(errcode(ERRCODE_DATA_EXCEPTION),
 				 errmsg("password \"%s\" is weak", str)));
 
+		PG_RETURN_POINTER(NULL);
+	}
+
 	result = (chkpass *) palloc(sizeof(chkpass));
 
 	mysalt[0] = salt_chars[random() & 0x3f];
@@ -108,6 +112,9 @@ chkpass_out(PG_FUNCTION_ARGS)
 	chkpass    *password = (chkpass *) PG_GETARG_POINTER(0);
 	char	   *result;
 
+	if (password == NULL)
+		PG_RETURN_POINTER(NULL);
+
 	if ((result = (char *) palloc(16)) != NULL)
 	{
 		result[0] = ':';
@@ -127,7 +134,10 @@ Datum
 chkpass_rout(PG_FUNCTION_ARGS)
 {
 	chkpass    *password = (chkpass *) PG_GETARG_POINTER(0);
-	text	   *result;
+	text	   *result = NULL;
+
+	if (password == NULL)
+		PG_RETURN_POINTER(NULL);
 
 	if ((result = (text *) palloc(VARHDRSZ + 16)) != NULL)
 	{
@@ -152,6 +162,9 @@ chkpass_eq(PG_FUNCTION_ARGS)
 	char		str[10];
 	int			sz = 8;
 
+	if (!a1 || !a2)
+		PG_RETURN_BOOL(0);
+
 	if (a2->vl_len < 12)
 		sz = a2->vl_len - 4;
 	strncpy(str, a2->vl_dat, sz);
@@ -168,6 +181,8 @@ chkpass_ne(PG_FUNCTION_ARGS)
 	char		str[10];
 	int			sz = 8;
 
+	if (!a1 || !a2)
+		PG_RETURN_BOOL(0);
 	if (a2->vl_len < 12)
 		sz = a2->vl_len - 4;
 	strncpy(str, a2->vl_dat, sz);
