@@ -7,11 +7,11 @@
  * the nature and use of path keys.
  *
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/optimizer/path/pathkeys.c,v 1.53.4.1 2003/12/03 17:45:36 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/optimizer/path/pathkeys.c,v 1.63 2004/12/31 22:00:04 pgsql Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -48,10 +48,11 @@ makePathKeyItem(Node *key, Oid sortop, bool checkType)
 
 	/*
 	 * Some callers pass expressions that are not necessarily of the same
-	 * type as the sort operator expects as input (for example when dealing
-	 * with an index that uses binary-compatible operators).  We must relabel
-	 * these with the correct type so that the key expressions will be seen
-	 * as equal() to expressions that have been correctly labeled.
+	 * type as the sort operator expects as input (for example when
+	 * dealing with an index that uses binary-compatible operators).  We
+	 * must relabel these with the correct type so that the key
+	 * expressions will be seen as equal() to expressions that have been
+	 * correctly labeled.
 	 */
 	if (checkType)
 	{
@@ -95,8 +96,8 @@ add_equijoined_keys(Query *root, RestrictInfo *restrictinfo)
 	PathKeyItem *item2 = makePathKeyItem(get_rightop(clause),
 										 restrictinfo->right_sortop,
 										 false);
-	List	   *newset,
-			   *cursetlink;
+	List	   *newset;
+	ListCell   *cursetlink;
 
 	/* We might see a clause X=X; don't make a single-element list from it */
 	if (equal(item1, item2))
@@ -122,12 +123,12 @@ add_equijoined_keys(Query *root, RestrictInfo *restrictinfo)
 	newset = NIL;
 
 	/* cannot use foreach here because of possible lremove */
-	cursetlink = root->equi_key_list;
+	cursetlink = list_head(root->equi_key_list);
 	while (cursetlink)
 	{
-		List	   *curset = lfirst(cursetlink);
-		bool		item1here = member(item1, curset);
-		bool		item2here = member(item2, curset);
+		List	   *curset = (List *) lfirst(cursetlink);
+		bool		item1here = list_member(curset, item1);
+		bool		item2here = list_member(curset, item2);
 
 		/* must advance cursetlink before lremove possibly pfree's it */
 		cursetlink = lnext(cursetlink);
@@ -147,22 +148,22 @@ add_equijoined_keys(Query *root, RestrictInfo *restrictinfo)
 
 			/* Build the new set only when we know we must */
 			if (newset == NIL)
-				newset = makeList2(item1, item2);
+				newset = list_make2(item1, item2);
 
 			/* Found a set to merge into our new set */
-			newset = set_union(newset, curset);
+			newset = list_union(newset, curset);
 
 			/*
 			 * Remove old set from equi_key_list.
 			 */
-			root->equi_key_list = lremove(curset, root->equi_key_list);
-			freeList(curset);	/* might as well recycle old cons cells */
+			root->equi_key_list = list_delete_ptr(root->equi_key_list, curset);
+			list_free(curset);	/* might as well recycle old cons cells */
 		}
 	}
 
 	/* Build the new set only when we know we must */
 	if (newset == NIL)
-		newset = makeList2(item1, item2);
+		newset = list_make2(item1, item2);
 
 	root->equi_key_list = lcons(newset, root->equi_key_list);
 }
@@ -199,15 +200,15 @@ add_equijoined_keys(Query *root, RestrictInfo *restrictinfo)
 void
 generate_implied_equalities(Query *root)
 {
-	List	   *cursetlink;
+	ListCell   *cursetlink;
 
 	foreach(cursetlink, root->equi_key_list)
 	{
-		List	   *curset = lfirst(cursetlink);
-		int			nitems = length(curset);
+		List	   *curset = (List *) lfirst(cursetlink);
+		int			nitems = list_length(curset);
 		Relids	   *relids;
 		bool		have_consts;
-		List	   *ptr1;
+		ListCell   *ptr1;
 		int			i1;
 
 		/*
@@ -245,10 +246,10 @@ generate_implied_equalities(Query *root)
 		{
 			PathKeyItem *item1 = (PathKeyItem *) lfirst(ptr1);
 			bool		i1_is_variable = !bms_is_empty(relids[i1]);
-			List	   *ptr2;
+			ListCell   *ptr2;
 			int			i2 = i1 + 1;
 
-			foreach(ptr2, lnext(ptr1))
+			for_each_cell(ptr2, lnext(ptr1))
 			{
 				PathKeyItem *item2 = (PathKeyItem *) lfirst(ptr2);
 				bool		i2_is_variable = !bms_is_empty(relids[i2]);
@@ -294,14 +295,14 @@ generate_implied_equalities(Query *root)
 bool
 exprs_known_equal(Query *root, Node *item1, Node *item2)
 {
-	List	   *cursetlink;
+	ListCell   *cursetlink;
 
 	foreach(cursetlink, root->equi_key_list)
 	{
-		List	   *curset = lfirst(cursetlink);
+		List	   *curset = (List *) lfirst(cursetlink);
 		bool		item1member = false;
 		bool		item2member = false;
-		List	   *ptr;
+		ListCell   *ptr;
 
 		foreach(ptr, curset)
 		{
@@ -334,17 +335,17 @@ exprs_known_equal(Query *root, Node *item1, Node *item2)
 static List *
 make_canonical_pathkey(Query *root, PathKeyItem *item)
 {
-	List	   *cursetlink;
 	List	   *newset;
+	ListCell   *cursetlink;
 
 	foreach(cursetlink, root->equi_key_list)
 	{
-		List	   *curset = lfirst(cursetlink);
+		List	   *curset = (List *) lfirst(cursetlink);
 
-		if (member(item, curset))
+		if (list_member(curset, item))
 			return curset;
 	}
-	newset = makeList1(item);
+	newset = list_make1(item);
 	root->equi_key_list = lcons(newset, root->equi_key_list);
 	return newset;
 }
@@ -360,11 +361,11 @@ List *
 canonicalize_pathkeys(Query *root, List *pathkeys)
 {
 	List	   *new_pathkeys = NIL;
-	List	   *i;
+	ListCell   *l;
 
-	foreach(i, pathkeys)
+	foreach(l, pathkeys)
 	{
-		List	   *pathkey = (List *) lfirst(i);
+		List	   *pathkey = (List *) lfirst(l);
 		PathKeyItem *item;
 		List	   *cpathkey;
 
@@ -374,7 +375,7 @@ canonicalize_pathkeys(Query *root, List *pathkeys)
 		 * set by definition.
 		 */
 		Assert(pathkey != NIL);
-		item = (PathKeyItem *) lfirst(pathkey);
+		item = (PathKeyItem *) linitial(pathkey);
 		cpathkey = make_canonical_pathkey(root, item);
 
 		/*
@@ -383,7 +384,7 @@ canonicalize_pathkeys(Query *root, List *pathkeys)
 		 * canonicalized the keys, so that equivalent-key knowledge is
 		 * used when deciding if an item is redundant.
 		 */
-		if (!ptrMember(cpathkey, new_pathkeys))
+		if (!list_member_ptr(new_pathkeys, cpathkey))
 			new_pathkeys = lappend(new_pathkeys, cpathkey);
 	}
 	return new_pathkeys;
@@ -402,14 +403,14 @@ canonicalize_pathkeys(Query *root, List *pathkeys)
 static int
 count_canonical_peers(Query *root, PathKeyItem *item)
 {
-	List	   *cursetlink;
+	ListCell   *cursetlink;
 
 	foreach(cursetlink, root->equi_key_list)
 	{
-		List	   *curset = lfirst(cursetlink);
+		List	   *curset = (List *) lfirst(cursetlink);
 
-		if (member(item, curset))
-			return length(curset) - 1;
+		if (list_member(curset, item))
+			return list_length(curset) - 1;
 	}
 	return 0;
 }
@@ -430,23 +431,21 @@ count_canonical_peers(Query *root, PathKeyItem *item)
 PathKeysComparison
 compare_pathkeys(List *keys1, List *keys2)
 {
-	List	   *key1,
+	ListCell   *key1,
 			   *key2;
 
-	for (key1 = keys1, key2 = keys2;
-		 key1 != NIL && key2 != NIL;
-		 key1 = lnext(key1), key2 = lnext(key2))
+	forboth(key1, keys1, key2, keys2)
 	{
-		List	   *subkey1 = lfirst(key1);
-		List	   *subkey2 = lfirst(key2);
+		List	   *subkey1 = (List *) lfirst(key1);
+		List	   *subkey2 = (List *) lfirst(key2);
 
 		/*
 		 * XXX would like to check that we've been given canonicalized
 		 * input, but query root not accessible here...
 		 */
 #ifdef NOT_USED
-		Assert(ptrMember(subkey1, root->equi_key_list));
-		Assert(ptrMember(subkey2, root->equi_key_list));
+		Assert(list_member_ptr(root->equi_key_list, subkey1));
+		Assert(list_member_ptr(root->equi_key_list, subkey2));
 #endif
 
 		/*
@@ -465,9 +464,9 @@ compare_pathkeys(List *keys1, List *keys2)
 	 * the other list are not NIL --- no pathkey list should ever have a
 	 * NIL sublist.)
 	 */
-	if (key1 == NIL && key2 == NIL)
+	if (key1 == NULL && key2 == NULL)
 		return PATHKEYS_EQUAL;
-	if (key1 != NIL)
+	if (key1 != NULL)
 		return PATHKEYS_BETTER1;	/* key1 is longer */
 	return PATHKEYS_BETTER2;	/* key2 is longer */
 }
@@ -493,18 +492,16 @@ compare_pathkeys(List *keys1, List *keys2)
 PathKeysComparison
 compare_noncanonical_pathkeys(List *keys1, List *keys2)
 {
-	List	   *key1,
+	ListCell   *key1,
 			   *key2;
 
-	for (key1 = keys1, key2 = keys2;
-		 key1 != NIL && key2 != NIL;
-		 key1 = lnext(key1), key2 = lnext(key2))
+	forboth(key1, keys1, key2, keys2)
 	{
-		List	   *subkey1 = lfirst(key1);
-		List	   *subkey2 = lfirst(key2);
+		List	   *subkey1 = (List *) lfirst(key1);
+		List	   *subkey2 = (List *) lfirst(key2);
 
-		Assert(length(subkey1) == 1);
-		Assert(length(subkey2) == 1);
+		Assert(list_length(subkey1) == 1);
+		Assert(list_length(subkey2) == 1);
 		if (!equal(subkey1, subkey2))
 			return PATHKEYS_DIFFERENT;	/* no need to keep looking */
 	}
@@ -515,9 +512,9 @@ compare_noncanonical_pathkeys(List *keys1, List *keys2)
 	 * the other list are not NIL --- no pathkey list should ever have a
 	 * NIL sublist.)
 	 */
-	if (key1 == NIL && key2 == NIL)
+	if (key1 == NULL && key2 == NULL)
 		return PATHKEYS_EQUAL;
-	if (key1 != NIL)
+	if (key1 != NULL)
 		return PATHKEYS_BETTER1;	/* key1 is longer */
 	return PATHKEYS_BETTER2;	/* key2 is longer */
 }
@@ -573,11 +570,11 @@ get_cheapest_path_for_pathkeys(List *paths, List *pathkeys,
 							   CostSelector cost_criterion)
 {
 	Path	   *matched_path = NULL;
-	List	   *i;
+	ListCell   *l;
 
-	foreach(i, paths)
+	foreach(l, paths)
 	{
-		Path	   *path = (Path *) lfirst(i);
+		Path	   *path = (Path *) lfirst(l);
 
 		/*
 		 * Since cost comparison is a lot cheaper than pathkey comparison,
@@ -612,11 +609,11 @@ get_cheapest_fractional_path_for_pathkeys(List *paths,
 										  double fraction)
 {
 	Path	   *matched_path = NULL;
-	List	   *i;
+	ListCell   *l;
 
-	foreach(i, paths)
+	foreach(l, paths)
 	{
-		Path	   *path = (Path *) lfirst(i);
+		Path	   *path = (Path *) lfirst(l);
 
 		/*
 		 * Since cost comparison is a lot cheaper than pathkey comparison,
@@ -658,7 +655,7 @@ build_index_pathkeys(Query *root,
 	List	   *retval = NIL;
 	int		   *indexkeys = index->indexkeys;
 	Oid		   *ordering = index->ordering;
-	List	   *indexprs = index->indexprs;
+	ListCell   *indexprs_item = list_head(index->indexprs);
 
 	while (*ordering != InvalidOid)
 	{
@@ -683,10 +680,10 @@ build_index_pathkeys(Query *root,
 		else
 		{
 			/* expression --- assume we need not copy it */
-			if (indexprs == NIL)
+			if (indexprs_item == NULL)
 				elog(ERROR, "wrong number of index expressions");
-			indexkey = (Node *) lfirst(indexprs);
-			indexprs = lnext(indexprs);
+			indexkey = (Node *) lfirst(indexprs_item);
+			indexprs_item = lnext(indexprs_item);
 		}
 
 		/* OK, make a sublist for this sort key */
@@ -697,7 +694,7 @@ build_index_pathkeys(Query *root,
 		 * Eliminate redundant ordering info; could happen if query is
 		 * such that index keys are equijoined...
 		 */
-		if (!ptrMember(cpathkey, retval))
+		if (!list_member_ptr(retval, cpathkey))
 			retval = lappend(retval, cpathkey);
 
 		indexkeys++;
@@ -719,13 +716,13 @@ build_index_pathkeys(Query *root,
 static Var *
 find_indexkey_var(Query *root, RelOptInfo *rel, AttrNumber varattno)
 {
-	List	   *temp;
+	ListCell   *temp;
 	Index		relid;
 	Oid			reloid,
 				vartypeid;
 	int32		type_mod;
 
-	foreach(temp, FastListValue(&rel->reltargetlist))
+	foreach(temp, rel->reltargetlist)
 	{
 		Var		   *var = (Var *) lfirst(temp);
 
@@ -756,13 +753,14 @@ build_subquery_pathkeys(Query *root, RelOptInfo *rel, Query *subquery)
 {
 	List	   *retval = NIL;
 	int			retvallen = 0;
-	int			outer_query_keys = length(root->query_pathkeys);
-	List	   *l;
+	int			outer_query_keys = list_length(root->query_pathkeys);
+	List	   *sub_tlist = rel->subplan->targetlist;
+	ListCell   *i;
 
-	foreach(l, subquery->query_pathkeys)
+	foreach(i, subquery->query_pathkeys)
 	{
-		List	   *sub_pathkey = (List *) lfirst(l);
-		List	   *j;
+		List	   *sub_pathkey = (List *) lfirst(i);
+		ListCell   *j;
 		PathKeyItem *best_item = NULL;
 		int			best_score = 0;
 		List	   *cpathkey;
@@ -787,9 +785,9 @@ build_subquery_pathkeys(Query *root, RelOptInfo *rel, Query *subquery)
 		{
 			PathKeyItem *sub_item = (PathKeyItem *) lfirst(j);
 			Node	   *sub_key = sub_item->key;
-			List	   *k;
+			ListCell   *k;
 
-			foreach(k, subquery->targetList)
+			foreach(k, sub_tlist)
 			{
 				TargetEntry *tle = (TargetEntry *) lfirst(k);
 
@@ -813,8 +811,7 @@ build_subquery_pathkeys(Query *root, RelOptInfo *rel, Query *subquery)
 					score = count_canonical_peers(root, outer_item);
 					/* +1 if it matches the proper query_pathkeys item */
 					if (retvallen < outer_query_keys &&
-						member(outer_item,
-							   nth(retvallen, root->query_pathkeys)))
+						list_member(list_nth(root->query_pathkeys, retvallen), outer_item))
 						score++;
 					if (score > best_score)
 					{
@@ -839,7 +836,7 @@ build_subquery_pathkeys(Query *root, RelOptInfo *rel, Query *subquery)
 		 * Eliminate redundant ordering info; could happen if outer query
 		 * equijoins subquery keys...
 		 */
-		if (!ptrMember(cpathkey, retval))
+		if (!list_member_ptr(retval, cpathkey))
 		{
 			retval = lappend(retval, cpathkey);
 			retvallen++;
@@ -907,11 +904,11 @@ make_pathkeys_for_sortclauses(List *sortclauses,
 							  List *tlist)
 {
 	List	   *pathkeys = NIL;
-	List	   *i;
+	ListCell   *l;
 
-	foreach(i, sortclauses)
+	foreach(l, sortclauses)
 	{
-		SortClause *sortcl = (SortClause *) lfirst(i);
+		SortClause *sortcl = (SortClause *) lfirst(l);
 		Node	   *sortkey;
 		PathKeyItem *pathkey;
 
@@ -923,7 +920,7 @@ make_pathkeys_for_sortclauses(List *sortclauses,
 		 * canonicalize_pathkeys() might replace it with a longer sublist
 		 * later.
 		 */
-		pathkeys = lappend(pathkeys, makeList1(pathkey));
+		pathkeys = lappend(pathkeys, list_make1(pathkey));
 	}
 	return pathkeys;
 }
@@ -999,21 +996,21 @@ find_mergeclauses_for_pathkeys(Query *root,
 							   List *restrictinfos)
 {
 	List	   *mergeclauses = NIL;
-	List	   *i;
+	ListCell   *i;
 
 	/* make sure we have pathkeys cached in the clauses */
 	foreach(i, restrictinfos)
 	{
-		RestrictInfo *restrictinfo = lfirst(i);
+		RestrictInfo *restrictinfo = (RestrictInfo *) lfirst(i);
 
 		cache_mergeclause_pathkeys(root, restrictinfo);
 	}
 
 	foreach(i, pathkeys)
 	{
-		List	   *pathkey = lfirst(i);
+		List	   *pathkey = (List *) lfirst(i);
 		List	   *matched_restrictinfos = NIL;
-		List	   *j;
+		ListCell   *j;
 
 		/*
 		 * We can match a pathkey against either left or right side of any
@@ -1036,7 +1033,7 @@ find_mergeclauses_for_pathkeys(Query *root,
 		 */
 		foreach(j, restrictinfos)
 		{
-			RestrictInfo *restrictinfo = lfirst(j);
+			RestrictInfo *restrictinfo = (RestrictInfo *) lfirst(j);
 
 			/*
 			 * We can compare canonical pathkey sublists by simple pointer
@@ -1044,7 +1041,7 @@ find_mergeclauses_for_pathkeys(Query *root,
 			 */
 			if ((pathkey == restrictinfo->left_pathkey ||
 				 pathkey == restrictinfo->right_pathkey) &&
-				!ptrMember(restrictinfo, mergeclauses))
+				!list_member_ptr(mergeclauses, restrictinfo))
 			{
 				matched_restrictinfos = lappend(matched_restrictinfos,
 												restrictinfo);
@@ -1063,7 +1060,7 @@ find_mergeclauses_for_pathkeys(Query *root,
 		 * If we did find usable mergeclause(s) for this sort-key
 		 * position, add them to result list.
 		 */
-		mergeclauses = nconc(mergeclauses, matched_restrictinfos);
+		mergeclauses = list_concat(mergeclauses, matched_restrictinfos);
 	}
 
 	return mergeclauses;
@@ -1092,11 +1089,11 @@ make_pathkeys_for_mergeclauses(Query *root,
 							   RelOptInfo *rel)
 {
 	List	   *pathkeys = NIL;
-	List	   *i;
+	ListCell   *l;
 
-	foreach(i, mergeclauses)
+	foreach(l, mergeclauses)
 	{
-		RestrictInfo *restrictinfo = (RestrictInfo *) lfirst(i);
+		RestrictInfo *restrictinfo = (RestrictInfo *) lfirst(l);
 		List	   *pathkey;
 
 		cache_mergeclause_pathkeys(root, restrictinfo);
@@ -1127,7 +1124,7 @@ make_pathkeys_for_mergeclauses(Query *root,
 		 * pathkey, a simple ptrMember test is sufficient to detect
 		 * redundant keys.
 		 */
-		if (!ptrMember(pathkey, pathkeys))
+		if (!list_member_ptr(pathkeys, pathkey))
 			pathkeys = lappend(pathkeys, pathkey);
 	}
 
@@ -1159,18 +1156,18 @@ int
 pathkeys_useful_for_merging(Query *root, RelOptInfo *rel, List *pathkeys)
 {
 	int			useful = 0;
-	List	   *i;
+	ListCell   *i;
 
 	foreach(i, pathkeys)
 	{
-		List	   *pathkey = lfirst(i);
+		List	   *pathkey = (List *) lfirst(i);
 		bool		matched = false;
-		List	   *j;
+		ListCell   *j;
 
 		foreach(j, rel->joininfo)
 		{
 			JoinInfo   *joininfo = (JoinInfo *) lfirst(j);
-			List	   *k;
+			ListCell   *k;
 
 			foreach(k, joininfo->jinfo_restrictinfo)
 			{
@@ -1217,7 +1214,7 @@ pathkeys_useful_for_merging(Query *root, RelOptInfo *rel, List *pathkeys)
  *
  * Unlike merge pathkeys, this is an all-or-nothing affair: it does us
  * no good to order by just the first key(s) of the requested ordering.
- * So the result is always either 0 or length(root->query_pathkeys).
+ * So the result is always either 0 or list_length(root->query_pathkeys).
  */
 int
 pathkeys_useful_for_ordering(Query *root, List *pathkeys)
@@ -1231,7 +1228,7 @@ pathkeys_useful_for_ordering(Query *root, List *pathkeys)
 	if (pathkeys_contained_in(root->query_pathkeys, pathkeys))
 	{
 		/* It's useful ... or at least the first N keys are */
-		return length(root->query_pathkeys);
+		return list_length(root->query_pathkeys);
 	}
 
 	return 0;					/* path ordering not useful */
@@ -1258,8 +1255,8 @@ truncate_useless_pathkeys(Query *root,
 	 * Note: not safe to modify input list destructively, but we can avoid
 	 * copying the list if we're not actually going to change it
 	 */
-	if (nuseful == length(pathkeys))
+	if (nuseful == list_length(pathkeys))
 		return pathkeys;
 	else
-		return ltruncate(nuseful, listCopy(pathkeys));
+		return list_truncate(list_copy(pathkeys), nuseful);
 }
