@@ -36,10 +36,10 @@
  * to look like NO SCROLL cursors.
  *
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: portal.h,v 1.47.4.1 2010/07/05 09:27:56 heikki Exp $
+ * $PostgreSQL: pgsql/src/include/utils/portal.h,v 1.54 2004/12/31 22:03:46 pgsql Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -48,6 +48,7 @@
 
 #include "executor/execdesc.h"
 #include "nodes/memnodes.h"
+#include "utils/resowner.h"
 #include "utils/tuplestore.h"
 
 
@@ -79,6 +80,20 @@ typedef enum PortalStrategy
 	PORTAL_MULTI_QUERY
 } PortalStrategy;
 
+/*
+ * A portal is always in one of these states.  It is possible to transit
+ * from ACTIVE back to READY if the query is not run to completion;
+ * otherwise we never back up in status.
+ */
+typedef enum PortalStatus
+{
+	PORTAL_NEW,					/* in process of creation */
+	PORTAL_READY,				/* PortalStart complete, can run it */
+	PORTAL_ACTIVE,				/* portal is running (can't delete it) */
+	PORTAL_DONE,				/* portal is finished (don't re-run it) */
+	PORTAL_FAILED				/* portal got error (can't re-run it) */
+} PortalStatus;
+
 /*
  * Note: typedef Portal is declared in tcop/dest.h as
  *		typedef struct PortalData *Portal;
@@ -89,8 +104,13 @@ typedef struct PortalData
 	/* Bookkeeping data */
 	const char *name;			/* portal's name */
 	MemoryContext heap;			/* subsidiary memory for portal */
-	void		(*cleanup) (Portal portal, bool isError);		/* cleanup hook */
-	TransactionId createXact;	/* the xid of the creating xact */
+	ResourceOwner resowner;		/* resources owned by portal */
+	void		(*cleanup) (Portal portal);		/* cleanup hook */
+	SubTransactionId createSubid;	/* the ID of the creating subxact */
+	/*
+	 * if createSubid is InvalidSubTransactionId, the portal is held over
+	 * from a previous transaction
+	 */
 
 	/* The query or queries the portal will execute */
 	const char *sourceText;		/* text of query, if known (may be NULL) */
@@ -113,11 +133,8 @@ typedef struct PortalData
 	int			cursorOptions;	/* DECLARE CURSOR option bits */
 
 	/* Status data */
-	bool		portalReady;	/* PortalStart complete? */
+	PortalStatus status;		/* see above */
 	bool		portalUtilReady;	/* PortalRunUtility complete? */
-	bool		portalActive;	/* portal is running (can't delete it) */
-	bool		portalDone;		/* portal is finished (don't re-run it) */
-	bool		portalPinned;	/* portal is pinned (can't delete it) */
 
 	/* If not NULL, Executor is active; call ExecutorEnd eventually: */
 	QueryDesc  *queryDesc;		/* info needed for executor invocation */
@@ -168,11 +185,16 @@ extern void EnablePortalManager(void);
 extern void AtCommit_Portals(void);
 extern void AtAbort_Portals(void);
 extern void AtCleanup_Portals(void);
+extern void AtSubCommit_Portals(SubTransactionId mySubid,
+								SubTransactionId parentSubid,
+								ResourceOwner parentXactOwner);
+extern void AtSubAbort_Portals(SubTransactionId mySubid,
+							   SubTransactionId parentSubid,
+							   ResourceOwner parentXactOwner);
+extern void AtSubCleanup_Portals(SubTransactionId mySubid);
 extern Portal CreatePortal(const char *name, bool allowDup, bool dupSilent);
 extern Portal CreateNewPortal(void);
-extern void PinPortal(Portal portal);
-extern void UnpinPortal(Portal portal);
-extern void PortalDrop(Portal portal, bool isError);
+extern void PortalDrop(Portal portal, bool isTopCommit);
 extern void DropDependentPortals(MemoryContext queryContext);
 extern Portal GetPortalByName(const char *name);
 extern void PortalDefineQuery(Portal portal,
